function [x, pos, nr, hdst, time, ok] = lynxgeng(filename, channel, pos, n)
%LYNXGENG Read samples from Lynx AqDados .TEM / LTD  file.
%
%   [X, POS, NR, HDST, TIME, OK] = LYNXGENG(FILENAME) reads all samples
%   of all channels of the Lynx AqDados .TEM file especified in FILENAME.
%   OK return -1 in the case of error or 0 in the case of success. X is matriz
%   with NR rows by number of channels cols. NR is the number of samples per
%   channels. Data from AqDados .tem file header is returned in HDST. In POS
%   is returned the new sample index position in the file. TIME is an array 
%   with time of each sample.
%
%   [X, POS, NR, HDST, TIME, OK] = LYNXGENG(FILENAME, CHANNEL) reads all
%   samples of the channel specified in the CHANNEL parameter. You must
%   specified 1 in CHANNEL to read the first active channel of the file, 2
%   to read the second active channel, and so on. When 0 or -1 is passed to 
%   this function, all active channel will be read.
%
%   [X, POS, NR, HDST, TIME, OK] = LYNXGENG(FILENAME, CHANNEL, POS, N)
%   reads N samples of the channel specified in the CHANNEL parameter started
%   by POS-th sample of the file. If 0 or -1 value is passed to N parameter,
%   the remainder samples in the file will be read. For example, to read 256
%   samples of the second channel begining from 1024th sample use the
%   command:
%
%   [x, pos, nr] = lynxgeng ('demodado.tem', 2, 1024, 256);
%   [x, pos, nr] = lynxgeng ('demodado.ltd', 2, 1024, 256);
%
%   See LYNXINFO to get HDST struct format.
%
%   See also LYNXINFO.

if (nargin == 1)
    channel = 0;
    pos = 0;
    n = 0;
elseif (nargin == 2)
    pos = 0;
    n = 0;
elseif (nargin == 3)
    n = 0;
elseif (nargin == 4)
else
    error('Incorrect number of input arguments.');
    return;
end;

if (channel < 0) channel = 0; end;
if (pos < 0) pos = 0; end;
if (n < 0) n = 0; end;

if ~isstr(filename)
    error('Filename must be a string argument!');
    return
end

[f_path, f_name, f_ext] = fileparts (upper(filename));

switch f_ext
    case '.TEM'
        % --- Read header data from AqDados .TEM file ---
        [hdst, ok] = fileheader_tem (filename);
        if ok == -1
          error('Cannot open AqDados file!');
          return
        end
        
    case {'.LTD', '.LTX'}
        % --- Read header data from AqDados .LTD or .LTX file ---
        [hdst, ok] = fileheader_ts (filename);
        if ok == -1
          error('Cannot open AqDados file!');
          return
        end
        
    otherwise
        error('Invalid file type!');
        return
end

% --- check channel number ---
if (channel > hdst.nc)
    error('Invalid channel!');
    return
end;

% --- check pos ---
if (pos >= hdst.ns)
    error('lynxgeng: no more samples!');
    return
end;

% --- Open AqDados file ---
hd = fopen (filename, 'r');
if hd == -1
    error('Cannot open AqDados file!');
    return
end

% === Read all channels ===
nc = hdst.nc;

% === sample position ===
p = pos * nc * hdst.SampleSize + hdst.FileOfs;

fseek (hd, p, -1);
if (n == 0)
    nr = hdst.ns - pos;
else
    nr = hdst.ns - pos;
    if (n < nr)
        nr = n;
    end;
end;
pos = pos + nr;
x = fread (hd, [nc, nr], hdst.SampleFormat);
x = x.';

% === engineering unit conversion factors ===
m = (hdst.lsup - hdst.linf) / hdst.BinRange;
b = m * hdst.BinRange / 2 + hdst.linf;

if (channel == 0)
    % --- convert all channels ---
    for j = 1 : hdst.nc,
        x(:,j) = x(:,j) * m(j) + b(j);
    end;
else
    % --- convert select channel ---
    x = x(:,channel) * m(channel) + b(channel);
end;

% --- process time array ---
if (nargout >= 5)
    dt = 1 / hdst.fs;
    time=[pos-nr:pos-1]*dt;
    time = time.';
end;

ok = fclose (hd);

