function [hdst, ok] = lynxinfo(filename)
%LYNXINFO Read Lynx AqDados .TEM file header.
%   [HDST, OK] = LYNXINFO(FILENAME) reads data from header of the
%   Lynx AqDados .TEM file especified in FILENAME. OK return -1 in the
%   case of error or 0 in the case of success. Lynx AqDados .TEM header
%   data  is returned in HDST, that is a struct type with the following
%   fields:
%       DTHR: 7 elements array with date and time of the acquisition.
%             dthr (1): day
%             dthr (2): month
%             dthr (3): year
%             dthr (4): hour
%             dthr (5): minute
%             dthr (6): second
%             dthr (7): ms
%       COMMENT: string with acquisition comment.
%       NC: number of channels
%       FS: sample frequency in Hz
%       NS: number of samples per channel
%       NAME: array of NC elements with channels names
%       UNIT: array of NC elements with channels engineering units
%       SUP: array of NC elements with A/D channels upper limit in
%            engineering unit
%       INF: array of NC elements with A/D channels lower limit in
%            engineering unit
%       NE: number of events
%       EV: array of NE elements with event "time" (in sample index)
%
%   To access data in the hdst struct is very simple. For exemplo, the
%   name of first active channel is accessed by:
%       hdst.name(1)
%
%   The list of channels returned in NAME, UNIT, SUP and INF correspond
%   only to active channel in the .TEM file.    
%
%   See also LYNXGENG.

if (nargin ~= 1)
    error('Invalid sintaxe! Use help lynxinfo');
    return
end;

if ~isstr(filename)
    error('Filename must be a string argument!');
    return
end

% --- Open AqDados .TEM file ---
hd = fopen (filename, 'r');
if hd == -1
    error('Cannot open AqDados file!');
    return
end

% --- Get acquisition date/time ---
dthr = fread (hd, 7, 'uint16');

% --- Get acquisition comment ---
n = fread (hd, 1, 'uint8');
dummy = fread (hd, [1,30], 'char');
comment = char (dummy(1:n));

%
% === Loop to get channels data ===
%
name = {};
unit = {};
ns   = 0;
for i=1:32,
    % --- Get channel signal name ---
    n = fread (hd, 1, 'uint8');
    dummy = fread (hd, [1,15], 'char');
    aname = char (dummy(1:n));
    % ---  Get channel signal unit ---
    n = fread (hd, 1, 'uint8');
    dummy = fread (hd, [1,7], 'char');
    aunit = char (dummy(1:n));
    % --- Get A/D channel signal eu upper limit ---
    asup = fread (hd, 1, 'float32');
    % --- Get A/D channel signal eu lower limit ---
    ainf = fread (hd, 1, 'float32');
    % --- Get A/D channel signal eu lower limit ---
    used = fread (hd, 1, 'uchar');
    fseek (hd, 2, 0);
    % --- Insert channel if is an active channel ---
    if used ~= 0
        ns = ns + 1;
        name = [name, {aname}];
        unit = [unit, {aunit}];
        sup (ns) = asup;
        inf (ns) = ainf;
    end;
end;

% --- Get the number of acquired channels ---
nc = fread (hd, 1, 'uint16');

% --- Get the sample frequency ---
fseek (hd, 4, 0);
fs = fread (hd, 1, 'float32');

% --- Get the number of samples acquired by channel ---
ns = fread (hd, 1, 'uint32');

% === Loop to get extra comments ===
%
fseek (hd, 252, 0);
excom = {};
for i=1:6,
    % --- Get i-th extra comment ---
    n = fread (hd, 1, 'uint8');
    dummy = fread (hd, [1,30], 'char');
    acom = char (dummy(1:n));
    excom = [excom, {acom}];
end;

% --- Get the number of events ---
ne = fread (hd, 1, 'uint16');

% --- Get the array of events ---
ev = fread (hd, ne, 'uint32');

% --- Create the structure to store the header data ---
hdst = struct ('dthr', dthr, 'comment', comment, 'nc', nc, 'fs', fs, 'ns', ns, ...
               'sname', {name}, 'sunit', {unit}, 'sup', sup, 'inf', inf, ...
               'excom', {excom}, 'ne', ne, 'ev', ev);

ok = fclose (hd);

