function [x, pos, nr, hdst, time, ok] = lynxturbinado(filename, channel, pos, n)
%LYNXTURBINADO Read samples from Lynx AqDados .TEM file.
%
%  contribution from Adriano Axel(adriano.axel@gmail.com) - IPT Sao Paulo - SP
%  %
%   [X, POS, NR, HDST, TIME, OK] = LYNXTURBINADO(FILENAME) reads all samples
%   of all channels of the Lynx AqDados .TEM file especified in FILENAME.
%   OK return -1 in the case of error or 0 in the case of success. X is matriz
%   with NR rows by number of channels cols. NR is the number of samples per
%   channels. Data from AqDados .tem file header is returned in HDST. In POS
%   is returned the new sample index position in the file. TIME is an array 
%   with time of each sample.
%
%   [X, POS, NR, HDST, TIME, OK] = LYNXGENG(FILENAME, CHANNEL) reads all
%   samples of the channel specified in the CHANNEL parameter. You must
%   specified 1 in CHANNEL to read the first active channel of the file, 2
%   to read the second active channel, and so on. When 0 or -1 is passed to 
%   this function, all active channel will be read.
%
%   [X, POS, NR, HDST, TIME, OK] = LYNXGENG(FILENAME, CHANNEL, POS, N)
%   reads N samples of the channel specified in the CHANNEL parameter started
%   by POS-th sample of the file. If 0 or -1 value is passed to N parameter,
%   the remainder samples in the file will be read. For example, to read 256
%   samples of the second channel begining from 1024th sample use the
%   command:
%
%   [x, pos, nr] = lynxgeng ('demodado.tem', 2, 1024, 256);
%
%   See LYNXINFO to get HDST struct format.
%
%   See also LYNXINFO.

if (nargin == 1)
    channel = 0;
    pos = 0;
    n = 0;
elseif (nargin == 2)
    pos = 0;
    n = 0;
elseif (nargin == 3)
    n = 0;
elseif (nargin == 4)
else
    error('Incorrect number of input arguments.');
    return;
end;

if (channel < 0) channel = 0; end;
if (pos < 0) pos = 0; end;
if (n < 0) n = 0; end;

if ~isstr(filename)
    error('Filename must be a string argument!');
    return
end

% --- Read header data from AqDados .TEM file ---
[hdst, ok] = lynxinfo (filename);
if ok == -1
    error('Cannot open AqDados file!');
    return
end

% --- check channel number ---
if (channel > hdst.nc)
    error('Invalid channel!');
    return
end;

% --- check pos ---
if (pos >= hdst.ns)
    error('lynxgeng: no more samples!');
    return
end;

% --- Open AqDados .TEM file ---
hd = fopen (filename, 'r');
if hd == -1
    error('Cannot open AqDados file!');
    return
end

% === Read all channels ===
%
nc = hdst.nc;
p = pos * nc * 2 + 1747;
fseek (hd, p, -1);
if (n == 0)
    nr = hdst.ns - pos;
else
    nr = hdst.ns - pos;
    if (n < nr)
        nr = n;
    end;
end;
pos = pos + nr;
x = fread (hd, [nc, nr], 'int16');
x = x.';

% === engineering unit conversion factors ===
for j=1:hdst.nc,
    m (j) = (hdst.sup (j) - hdst.inf (j)) / 65536.0;
    b (j) = m (j) * 32768.0 + hdst.inf (j);
end;
if (channel == 0)
    % --- convert all channels ---
    for j=1:hdst.nc,
        for i=1:nr,
            x(i,j) = x(i,j) * m(j) + b(j);
        end;
    end;
else

    % --- convert select channel ---
    for i=1:nr,
        x(i,channel) = x(i,channel) * m(channel) + b(channel);
    end;

    x=x(1:nr,channel);
end;

% --- process time array ---
if (nargout >= 5)
    dt = 1 / hdst.fs;
  
%     for i=1:nr,
%         time(i) = dt * (pos-nr+i-1);
%     end;
time=[pos-nr:pos-1]*dt;
    time = time.';
end;

ok = fclose (hd);

